/*
 * Proyecto: test
 * Fichero: Form1.prg
 * Descripcin:
 * Autor:
 * Fecha: 08/11/2021
 */

#include "Xailer.ch"

#define BUFFER_SIZE        65535
#define MAX_LINE_LENGTH    1024

CLASS TForm1 FROM TForm

   COMPONENT oButton1
   COMPONENT oBevel1

   METHOD CreateForm()
   METHOD Button1Click( oSender )

ENDCLASS

#include "Form1.xfm"

//------------------------------------------------------------------------------

METHOD Button1Click( oSender ) CLASS TForm1

   LOCAL oBrw

   WITH OBJECT oBrw := TTextBrowse():New( ::oBevel1 )
      :nAlign := alCLIENT
      :Create()
      :cFile := ".\source\form1.prg"
   END WITH
RETURN Nil

//------------------------------------------------------------------------------

#include "fileio.ch"


CLASS TTextBrowse FROM TBrowse

PUBLISHED:
   PROPERTY cFile         INIT "" WRITE SetFile
   PROPERTY nMarqueeStyle INIT bmHIGHLROW
   PROPERTY lHeader       INIT .F.

PUBLIC:
   METHOD Create( oParent ) CONSTRUCTOR
   METHOD Free()

   METHOD IsEof() // --> lResult
   METHOD IsBof() // --> lResult
   METHOD BookMark( nPos ) // --> nId
   METHOD KeyNo( nPos ) // --> nResult
   METHOD KeyCount( nPos ) // --> nCount
   METHOD Refresh( lComplete )   // --> lResult
   METHOD Goto( nPos ) // --> Nil

RESERVED:
   DATA aLines
   DATA hFile
   DATA nLine, nLines INIT 0

   METHOD MoveTop()
   METHOD MoveBottom()
   METHOD Skip( nSkip )
   METHOD SetFile( cFile )  // --> Nil

ENDCLASS

//------------------------------------------------------------------------------

METHOD Create( oParent ) CLASS TTextBrowse

   ::lRecordSelector := .f.
   ::oFont := TFont():Create( "Courier New", 8, 0, 400 )
   ::Super:Create( oParent )

RETURN Self

//------------------------------------------------------------------------------

METHOD Free() CLASS TTextBrowse

   IF !Empty( ::hfile )
      FClose( ::hFile )
      ::hFile := NIL
   ENDIF

RETURN ::Super:Free()

//------------------------------------------------------------------------------

METHOD MoveTop() CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnGoTop()

   IF xRet == Nil
      ::nLine := 1
   ENDIF

RETURN Nil

//------------------------------------------------------------------------------

METHOD MoveBottom() CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnGoBottom()

   IF xRet == Nil
      ::nLine := ::nLines
   ENDIF

RETURN Nil

//------------------------------------------------------------------------------

METHOD Skip( nSkip ) CLASS TTextBrowse

   LOCAL xRet
   LOCAL nActPos

   DEFAULT nSkip TO 1

   xRet := ::OnSkip( nSkip )

   IF xRet == Nil
      nActPos := ::nLine
      ::nLine += nSkip
      ::nLine := Min( Max( ::nLine, 1 ), ::nLines )
      RETURN ::nLine - nActPos
   ENDIF

RETURN xRet

//------------------------------------------------------------------------------

METHOD IsEof() CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnEof()

   IF xRet == Nil
      RETURN ( ::nLine == ::nLines )
   ENDIF

RETURN xRet

//------------------------------------------------------------------------------

METHOD IsBof() CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnBof()

   IF xRet == Nil
      RETURN ( ::nLine == 1 )
   ENDIF

RETURN xRet

//------------------------------------------------------------------------------

METHOD BookMark( nPos ) CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnBookMark( nPos )

   IF xRet == Nil
      xRet := ::nLine
      IF nPos != Nil
         ::nLine := nPos
      ENDIF
   ENDIF

RETURN xRet

//------------------------------------------------------------------------------

METHOD KeyNo( nPos ) CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnKeyNo( nPos )

   IF xRet == Nil
      IF nPos != Nil
         ::nLine := nPos
      ELSE
         xRet := ::nLine
      ENDIF
   ENDIF

   IF nPos != Nil
      ::nKeyNo := nPos
   ELSE
      ::nKeyNo := xRet
   ENDIF

RETURN xRet

//------------------------------------------------------------------------------

METHOD GoTo( nPos ) CLASS TTextBrowse

   IF ::nLen < 1
      RETURN Nil
   ENDIF

   ::DrawLine()
   ::KeyNo( nPos )
   ::OnChange( .T. )
   ::SetVPos( ::nLine )
   ::Refresh( .F. )

RETURN Nil

//------------------------------------------------------------------------------

METHOD KeyCount() CLASS TTextBrowse

   LOCAL xRet

   xRet := ::OnKeyCount()

   IF xRet == Nil
      xRet := ::nLines
   ENDIF

   ::nLen  := xRet
   ::nLine := Min( ::nLine, xRet )

RETURN xRet

//------------------------------------------------------------------------------

METHOD SetFile( cFile ) CLASS TTextBrowse

   IF !Empty( ::hfile )
      FClose( ::hFile )
   ENDIF

   ::hFile   := FOpen( cFile, FO_READ )
   ::alines  := GetLines( ::hFile )
   ::nLines  := Len( ::aLines )
   ::nLine   := 1
   ::nRowSel := 1

   ::Reset()

   TTextBrwColumn():New( Self ):Create()

   ::GetDisplayCols()

   IF ! Empty( ::Handle )
      ::aCols[ 1 ]:nWidth := MAX_LINE_LENGTH
      ::Refresh( .T. )
   ENDIF

RETURN cFile

//------------------------------------------------------------------------------

METHOD Refresh( lComplete )  CLASS TTextBrowse

   DEFAULT lComplete TO .F.

   IF lComplete
      ::nLine := 1
   ENDIF

RETURN ::Super:Refresh( lComplete )

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

CLASS TTextBrwColumn FROM TBrwColumn STATIC

RESERVED:
   METHOD GetData()
   METHOD SetData( Value, nLine ) VIRTUAL

ENDCLASS

//------------------------------------------------------------------------------

METHOD GetData() CLASS TTextBrwColumn

   LOCAL cText := Space( MAX_LINE_LENGTH )
   LOCAL nAt, nLen

   WITH OBJECT ::oParent
      IF :nLine > 0 .AND. :nLine <= :nLines
         FSeek( :hFile, :aLines[ :nLine ], FS_SET )
         nLen := FRead( :hFile, @cText, MAX_LINE_LENGTH )
         IF ( nAt := HB_At( CRLF, cText ) ) > 0
            cText := Left( cText, nAt )
         ELSE
            cText := Left( cText, nLen )
         ENDIF
         RETURN  cText
      ENDIF
   END WITH

RETURN ""

//------------------------------------------------------------------------------

STATIC FUNCTION GetLines( hFile )

   LOCAL aLines  := {}
   LOCAL cBuffer := Space( BUFFER_SIZE )
   LOCAL nPos, nAt, nTm

   AAdd( aLines, 0 )
   nPos := 0

   FSeek( hFile, nPos, FS_SET )

   WHILE FRead( hFile, @cBuffer, BUFFER_SIZE ) > 0
      nAt := 0
      nTm := 1
      DO WHILE ( nAt := HB_At( CRLF, cBuffer, nAt ) ) > 0
         AAdd( aLines, nAt + 1) // base 0. Solo incrementamos 1
         nAt += 2
         nTm := nAt
      ENDDO
      nPos += BUFFER_SIZE
      IF Right( cBuffer, 1 ) == CR
         FSeek( hFile, -1, FS_RELATIVE )
         nPos --
      ENDIF
   ENDDO

RETURN aLines

//------------------------------------------------------------------------------

